#
# spec file for package sapwmp
#
# Copyright (c) 2021 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define group_sapsys sapsys
Name:           sapwmp
Version:        0.1+git.1707179910.c6f5238
Release:        150400.3.3.1
Summary:        Configuration and utilities for collecting SAP processes under control group
License:        GPL-2.0-only
Group:          Productivity/Databases/Servers
%if 0%{?sle_version} == 150000
URL:            https://documentation.suse.com/sles-sap/15-GA/html/SLES4SAP-guide/cha-s4s-tune.html#sec-s4s-memory-protection
%endif
%if 0%{?sle_version} == 150100
URL:            https://documentation.suse.com/sles-sap/15-SP1/html/SLES4SAP-guide/cha-s4s-tune.html#sec-s4s-memory-protection
%endif
%if 0%{?sle_version} >= 150200
URL:            https://documentation.suse.com/sles-sap/15-SP2/html/SLES-SAP-guide/cha-tune.html#sec-memory-protection
%endif
Source0:        %{name}-%{version}.tar.xz
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  systemd-devel
BuildRequires:  systemd-rpm-macros
Requires(post): %fillup_prereq
Requires(post): permissions
Requires:       util-linux-systemd
# We need kernel fix for bsc#1174002
%if 0%{?sle_version} == 150200
Requires:       kernel >= 5.3.18-24.12
%endif
%{?systemd_requires}

%description
Configuration and utilities for collecting SAP processes under control group to apply resource control.

%prep
%setup -q

%build
./autogen.sh
%configure
# make_build not defined in SLE12, so need conditions to check
%if "x%{?make_build}" != "x"
%make_build
%else
%{__make} %{?jobs:-j%jobs}%{?!jobs:%{?_smp_mflags:%_smp_mflags}}
%endif


%install
%make_install
%define wmpd %{_builddir}/%{name}-%{version}
install -D -m 644 %{wmpd}/rpm/sapwmp.conf %{buildroot}/%{_sysconfdir}/sapwmp.conf
install -D -m 644 %{wmpd}/rpm/SAP.slice %{buildroot}/%{_unitdir}/SAP.slice
install -D -m 755 %{wmpd}/rpm/supportconfig-sapwmp %{buildroot}%{_prefix}/lib/supportconfig/plugins/sapwmp
install -D -m 744 %{wmpd}/rpm/wmp-sample-memory.sh %{buildroot}/%{_libexecdir}/sapwmp/wmp-sample-memory
install -D -m 644 %{wmpd}/rpm/wmp-sample-memory.service %{buildroot}/%{_unitdir}/wmp-sample-memory.service
install -D -m 644 %{wmpd}/rpm/wmp-sample-memory.timer %{buildroot}/%{_unitdir}/wmp-sample-memory.timer

mkdir -p %{buildroot}%{_sbindir}
install -D -m 755 %{wmpd}/scripts/wmp-check.sh %{buildroot}%{_sbindir}/wmp-check

mkdir -p %{buildroot}/%{_defaultdocdir}/sapwmp
install -D -m 644 %{wmpd}/scripts/README* %{buildroot}/%{_defaultdocdir}/sapwmp

%verifyscript
%verify_permissions -e %{_libexecdir}/sapwmp/sapwmp-capture

%pre
getent group %{group_sapsys} >/dev/null || cat <<EOD
The %{group_sapsys} group was not found! Most probably because no SAP software
is installed on your system. The ownership of some files are not sufficient
now.
To fix this, run the following commands after installing the SAP
software, which should create the group %{group_sapsys}:

chgrp %{group_sapsys} %{_libexecdir}/sapwmp/sapwmp-capture
chmod +s %{_libexecdir}/sapwmp/sapwmp-capture
EOD
%service_add_pre wmp-sample-memory.service wmp-sample-memory.timer

%post
%set_permissions %{_libexecdir}/sapwmp/sapwmp-capture
# Historically, we used 'sap.slice', check if there is any user configuration
# set with systemctl set-property and reassign to the current 'SAP.slice' (keep
# runtime copy for 'sap.slice').
# systemctl-daemon reload is implicit in the following service_add_post.
if [ "$1" -eq "2" -a -d %{_sysconfdir}/systemd/system.control/sap.slice.d ] ; then
	cp -r %{_sysconfdir}/systemd/system.control/sap.slice.d /run/systemd/system.control/sap.slice.d || :
	mv %{_sysconfdir}/systemd/system.control/sap.slice.d %{_sysconfdir}/systemd/system.control/SAP.slice.d \
	 && echo "Migrated configuration from sap.slice to SAP.slice"
fi
%service_add_post wmp-sample-memory.service wmp-sample-memory.timer
if grep -q " cgroup .*memory" /proc/mounts ; then
	echo "Warning: Found memory controller on v1 hierarchy. Make sure unified hierarchy only is used."
fi

%preun
%service_del_preun wmp-sample-memory.service wmp-sample-memory.timer

%postun
%service_del_postun wmp-sample-memory.service wmp-sample-memory.timer

%files
%defattr(-, root, root)
%dir %{_libexecdir}/sapwmp
%verify(not user group mode) %attr(4750,root,%{group_sapsys}) %{_libexecdir}/sapwmp/sapwmp-capture
%{_libexecdir}/sapwmp/wmp-sample-memory
%{_unitdir}/SAP.slice
%{_unitdir}/wmp-sample-memory.service
%{_unitdir}/wmp-sample-memory.timer
%config %{_sysconfdir}/sapwmp.conf
%dir %{_prefix}/lib/supportconfig
%dir %{_prefix}/lib/supportconfig/plugins
%{_prefix}/lib/supportconfig/plugins/sapwmp
%{_sbindir}/wmp-check
%{_defaultdocdir}/sapwmp

%changelog
* Tue Feb  6 2024 XLiang@suse.com
- Update to version 0.1+git.1707179910.c6f5238:
  * capture: Wait for StartTransientUnit() job finish (bsc#1217897)
  * capture: Rework verbose logging
  * Add a switch for disable OS detection.
* Tue Mar  1 2022 XLiang@suse.com
- Update to version 0.1+git.1645197740.6b06c5c:
  * wmp-check: Polish the phrase of error output.
  * wmp-check: raise error when user not configure MemoryLow of target slice in digital.
  * Polish pull request based on review comments.
  * check.sh: Add unprotect_list check of subcgroups
  * check.sh: Fix a wmp check bug of memory_low_children
  * Add switch f and avoid empty DBus message error.
  * Skip systemd managed processes jsc#PM-3309
* Fri Nov  5 2021 nwang@suse.com
- bsc#1192376, Update to version 0.1+git.1635770861.0992962:
  * Enable wmp-checker for SLE15SP4.
* Thu Apr 22 2021 nwang@suse.com
- Update to version 0.1+git.1619087532.40eb1b0:
  * Call sapwmp check in supportconfig.
  * bsc#1184865, remove sapstartsrv from process tree and enable SLE15SP3 wmp-check.
  * Install the wmp-check to /usr/sbin
  * spec: polish spec file based on review comments.
  * Modify the spec file to include script and remove duplicated files.
  * Include a script to check the setup of wmp.
  * capture: Graceful handling of missing PARENT_COMMANDS
  * capture: Apply scope limits only when they are non-zero
  * capture: Parse scope properties from config file
* Tue Sep  8 2020 mkoutny@suse.com
- Update to version 0.1+git.1599582034.723ec7d:
  * RPM: Update documentation URL
  * supportconfig: Adjust for generic slice name
  * RPM: Migrate configuration from sap.slice to SAP.slice
  * All: Rename sap.slice to SAP.slice
  * RPM: Don't treat sap.slice as service (bsc#1176264)
  * calibration: Row oriented format
* Fri Sep  4 2020 mkoutny@suse.com
- Update to version 0.1+git.1599229900.269a09c:
  * RPM: Require kernel fix for bsc#1174002
  * calibration: Randomize time of sampling
* Wed Aug 19 2020 mkoutny@suse.com
- Update to version 0.1+git.1597829321.6124d06cb94f:
  * cgroups: sap.slice has MemoryAccounting=yes (bsc#1175458)
  * supportconfig: Graceful handling of gone PIDs
  * Calibration: Make sure memory controller is enabled
  * RPM: Add better explanation of missing sapsys group
* Wed Jun  3 2020 mkoutny@suse.com
- Update to version 0.1+git.1591186085.937f404f2263:
  * RPM: Rename log to WMP
  * RPM: Install memory logging files
  * supportconfig: Capture some calibration data too
  * Calibration: Add memory measurement
  * RPM: specfile switch to master
  * RPM: Fix sap.slice documentation
* Tue May 19 2020 mkoutny@suse.com
- Switch to master git branch (rename only)
- Update to version 0.1+git.1589905944.e384d0ea1469:
* Wed Apr  8 2020 mkoutny@suse.com
- Update to version 0.1+git.1585821196.17801a880194:
  * supportconfig: Change method to list swapped processes
* Mon Mar 23 2020 mkoutny@suse.com
- Update to version 0.1+git.1584983560.8ebbcd5754ae:
  * Add supportconfig module
  * rpm: Unify path delimiters in .spec
* Mon Mar 16 2020 mkoutny@suse.com
- Update to version 0.1+git.1584386614.942e6694a1fa:
  * capture: Fix printing PIDs
  * capture: Do not fail with unknown options
  * capture: Add verbose logging
  * capture: Whitespace formatting
  * capture: Log empty capture too
* Mon Mar 16 2020 mkoutny@suse.com
- Update to version 0.1+git.1584120573.cfcc33f920e9:
  * capture: Print messages into syslog
  * capture: Refactor exit error logging
* Tue Mar 10 2020 mkoutny@suse.com
- Update to version 0.1+git.1583831247.962f274e98d7:
  * cgroup: Keep sapinit.service in default location
  * RPM: Clean with spec-cleaner
  * RPM: Apply packagers review suggestions
  * cgroup: Remove MemoryLow= from sap.slice
* Mon Feb 24 2020 mkoutny@suse.com
- Update to version 0.1+git.1582569440.b74788b4c549:
  * RPM: Move helper out of PATH
* Tue Feb 18 2020 mkoutny@suse.com
- Update to version 0.1+git.1582039625.7053418a2e47:
  * RPM: Clean spec file with format_spec_file
  * capture: Make error messages clearer
* Wed Feb 12 2020 mkoutny@suse.com
- Update to version 0.1+git.1581527222.d4f25ba0dab3:
  * RPM: Update URL to "usptream"
  * RPM: Remove Vendor: to satisfy checks
* Wed Feb 12 2020 mkoutny@suse.com
- Update to version 0.1+git.1581525375.77b22b8c50d2:
  * RPM: Strip -profile from package name
  * capture: Cancel TasksMax= limit
  * RPM: Add permission checks for setuid binary
  * capture: clang scan-build cleanup
  * RPM: Add slice description
  * capture: Simplify exit code value and logging
  * capture: Log MAX_PIDS limit hit
  * capture: Add action switch
  * capture: Add simple help
  * RPM: changelog generation
  * RPM: ignore absence of sapsys group before installation
* Wed Feb 12 2020 mkoutny@suse.com
- Package initialization
